from simpleMediaCenter.player.Omxplayer import Omxplayer
import logging
import os

class MPlayer(Omxplayer):
    __logger=logging.getLogger(__name__)
    __playerline="mplayer -slave -fs -vo xv -zoom"
    
    keymapping = {
        'pause' : 'pause\n',
        'quit'  : 'quit\n',
        'volume': {
                    'up'   : 'volume +5\n',
                    'down' : 'volume -5\n'
                  }
        }

    def __init__(self):
        self.__logger.debug("init")
    
    def getcmdline(self,file):
        return self.__playerline + " " + " '" + file + "'"
        




from simpleMediaCenter.player.Omxplayer import Omxplayer
from simpleMediaCenter.helpers.twitch import TwitchVideoResolver
import logging
import os

class Twitchplayer(Omxplayer):
    __logger=logging.getLogger(__name__)
    __cmdline="-o both"
    __playerline="omxplayer -b"
    __playlist="twitchplaylist.m3u8"
    

    def __init__(self, cmdline = "-o both"):
        self.__logger.debug("Twitchplayer init")
        self.__cmdline=cmdline
               
    
    def getcmdline(self,file):
        tvr = TwitchVideoResolver(self.__logger) 
        tvr.saveHLSToPlaylist(file, 0, self.__playlist)
        cmdline = self.__playerline + " " + self.__cmdline + " '" + self.__playlist + "'"
        self.__logger.debug('twitchplayer cmdline: ' + cmdline)
        return cmdline
        
    def stop(self):
        super(Twitchplayer, self).stop()
        if (os.path.isfile(self.__playlist)):
            os.remove(self.__playlist)
            



from simpleMediaCenter.interface.WebInterface import WebInterface, WebController
from simpleMediaCenter.interface.Interface import InterfaceListable
from simpleMediaCenter.browser.Browser import YoutubeBrowser
from simpleMediaCenter.playlist.Playlist import Single
import logging
import os
import sys
import configparser
import argparse

class SimpleMediaCenter():
    __logger=logging.getLogger(__name__)
    config_file = os.path.expanduser('~/.simpleMediaCenter-config.ini')
    config = None
    playerlist = None
    browserlist = None
    playlistlist = None
    controller = None
    interface = None
    
    '''
        init simpleMediaCenter, parse config
    '''
    def __init__(self):
        self.__logger.debug('init')
        self.config = configparser.ConfigParser()
        self.__logger.debug('setting default config')
        self.config = self.__setDefaultConfig(self.config)
        self.__logger.debug('trying to read config')
        try:
            self.__logger.info('using config file at ' + self.config_file)
            self.config.read_file(open(self.config_file))
        except FileNotFoundError as e:
            self.__logger.warning('config file not found, using default settings')
            try:
                self.__saveConfig()
            except Exception as e:
                self.__logger.critical('exception while saving config file: ' + repr(e))
        #parse config 
        self.__logger.info('parsing config')
        ##init players
        self.__logger.debug('parse players')
        array=[]
        try:
            if (self.config.getboolean('OMXPLAYER','use')):
                from simpleMediaCenter.player.Omxplayer import Omxplayer
                omxplayer = Omxplayer(self.config.get('OMXPLAYER','cmdline'))
                array.append(omxplayer)
            if (self.config.getboolean('TWITCHPLAYER','use')):
                from simpleMediaCenter.player.Twitchplayer import Twitchplayer
                twitchplayer = Twitchplayer(self.config.get('TWITCHPLAYER','cmdline'))
                array.append(twitchplayer)
            if (self.config.getboolean('YOUTUBEPLAYER','use')):
                from simpleMediaCenter.player.Youtubeplayer import Youtubeplayer
                youtubeplayer = Youtubeplayer(self.config.get('YOUTUBEPLAYER','cmdline'))
                array.append(youtubeplayer)
            if (self.config.getboolean('MPLAYER','use')):
                from simpleMediaCenter.player.MPlayer import MPlayer
                mplayer = MPlayer()
                array.append(mplayer)
        except ValueError as e:
            self.__logger.critical('value error while parsing players in  config file: ' + repr(e))
        self.playerlist = InterfaceListable(array)
        ##init browsers
        self.__logger.debug('parse browsers')
        array=[]
        try:
            if (self.config.getboolean('FILEBROWSER','use')):
                from simpleMediaCenter.browser.Browser import FileBrowser
                fileBrowser = FileBrowser()
                array.append(fileBrowser)
            if (self.config.getboolean('TWITCHBROWSER','use')):
                from simpleMediaCenter.browser.Browser import TwitchBrowser
                for username in self.config.get('TWITCHBROWSER','usernames').split(','):
                    tb = TwitchBrowser(username)
                    array.append(tb)
            if (self.config.getboolean('YOUTUBEBROWSER','use')):
                from simpleMediaCenter.browser.Browser import YoutubeBrowser
                youtubeBrowser = YoutubeBrowser()
                for favorite in self.config.get('YOUTUBEBROWSER','favorites').split(','):
                    youtubeBrowser.addFavorite(favorite)
                array.append(youtubeBrowser)            
        except ValueError as e:
            self.__logger.critical('value error while parsing players in  config file: ' + repr(e))
        self.browserlist = InterfaceListable(array)
        ##init playlists
        self.__logger.debug('parse playlists')
        array=[]
        try:
            if (self.config.getboolean('SINGLEPLAYLIST','use')):
                playList = Single()
                array.append(playList)          
        except ValueError as e:
            self.__logger.critical('value error while parsing players in  config file: ' + repr(e))
        self.playlistlist = InterfaceListable(array)
        ##create controller
        ##init webinterface
        self.__logger.debug('parse interface')
        try:
            if (self.config.getboolean('WEBINTERFACE','use')):
                pathToTemplates = self.config.get('WEBINTERFACE','templatePath')
                pathToStatic = self.config.get('WEBINTERFACE','staticPath')
                #TODO check if paths are valid
                port = self.config.getint('WEBINTERFACE', 'port')
                
                self.__logger.info('creating controller')
                self.controller = WebController(
                    playerList=self.playerlist, 
                    playlistList=self.playlistlist, 
                    browserList=self.browserlist)
                self.__logger.info('creating webinterface')
                self.interface = WebInterface(
                    pathToTemplates,
                    pathToStatic, 
                    port,
                    self.controller)
        
        except ValueError as e:
            self.__logger.critical('value error while parsing webinterface in  config file: ' + repr(e))
        except Exception as e:
            self.__logger.critical('error while parsing webinterface in  config file: ' + repr(e))
        #finished parsing
        
    '''
        run the media center by starting the previously setup interface
    '''
    def run(self):
        self.interface.run()

    def __setDefaultConfig(self, input_config):
        output_config = input_config
        
        #webinterface
        output_config['WEBINTERFACE'] = {
                'use'          : 'yes',
                'templatePath' : self.__getWebInterfacePath('templates/'),
                'staticPath'   : self.__getWebInterfacePath('static/'),    
                'port'         : '8080'
            }
        #players
        output_config['OMXPLAYER'] = {
                'use'     : 'yes',
                'cmdline' : '-o both'
            }
        output_config['TWITCHPLAYER'] = {
                'use'     : 'no',
                'cmdline' : '-o both'
            }
        output_config['YOUTUBEPLAYER'] = {
                'use'     : 'no',
                'cmdline' : '-o both'
            }
        output_config['MPLAYER'] = {
                'use'     : 'no'
            }
        #browser
        output_config['FILEBROWSER'] = {
                'use'   : 'yes'
            }
        output_config['TWITCHBROWSER'] = {
                'use'   : 'no',
                'usernames' : 'user1,user2'
            }
        output_config['YOUTUBEBROWSER'] = {
                'favorites' : 'MrSuicideSheep,Escapist,TopGear',
                'use'   : 'no'
            }
        #playlists
        output_config['SINGLEPLAYLIST'] = {
                'use'   : 'no'
            }
        
        return output_config
        
    def __saveConfig(self):
        with open(self.config_file, 'w') as configfile:
            self.config.write(configfile)
            
    def __getWebInterfacePath(self, subdir):
        path = os.path.abspath(os.path.dirname(sys.modules[WebInterface.__module__].__file__))
        path = os.path.abspath(os.path.join(path, subdir))
        logging.debug('getWebInterfacePath:' + path)
        return path
        
'''
    argument parsing
'''
def parseArgs():
    parser = argparse.ArgumentParser(
        description="simpleMediaCenter aims to provide an easy on ressources way to use your computer as an Media Center. It is primarily designed to act as an alternative to XBMC on the rpi.",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument('-D', '--debug', action='store_const',
                        const=logging.DEBUG, dest='verbosity',
                        help='Show all messages, including DEBUG')
    return parser.parse_args()



#-*- encoding: utf-8 -*-
import sys
try:
    from urllib.request import urlopen, Request
    from urllib.parse import quote_plus
except ImportError:
    from urllib import quote_plus
    from urllib2 import Request, urlopen

try:
    import json
except:
    import simplejson as json  # @UnresolvedImport

USER_AGENT = 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/20100101 Firefox/6.0'


class JSONScraper(object):
    '''
    Encapsulates execution request and parsing of response
    '''
    
    def __init__(self, logger):
        object.__init__(self)
        self.logger = logger
        
    '''
        Download Data from an url and returns it as a String
        @param url Url to download from (e.g. http://www.google.com)
        @param headers currently unused, backwards compability
        @returns String of data from URL
    '''
    def downloadWebData(self, url, headers=None):
        data = ""
        try:
            req = Request(url)
            req.add_header(Keys.USER_AGENT, USER_AGENT)
            response = urlopen(req)
            
            if sys.version_info < (3, 0):
                data = response.read()
            else:
                data = response.readall().decode('utf-8')
            response.close()
        except:
            raise TwitchException(TwitchException.HTTP_ERROR)
        return data
        
    '''
        Download Data from an url and returns it as JSON
        @param url Url to download from
        @param headers currently unused, backwards compability
        @returns JSON Object with data from URL
    '''
    def getJson(self, url, headers=None):
        try:
            jsonString = self.downloadWebData(url, headers)
        except:
            raise TwitchException(TwitchException.HTTP_ERROR)
        try:
            jsonDict = json.loads(jsonString)
            self.logger.debug(json.dumps(jsonDict, indent=4, sort_keys=True))
            return jsonDict
        except:
            raise TwitchException(TwitchException.JSON_ERROR)


class TwitchTV(object):
    '''
    Uses Twitch API to fetch json-encoded objects
    every method returns a dict containing the objects\' values
    '''
    def __init__(self, logger):
        self.logger = logger
        self.scraper = JSONScraper(logger)

    def getFeaturedStream(self):
        url = ''.join([Urls.STREAMS, Keys.FEATURED])
        return self._fetchItems(url, Keys.FEATURED)

    def getGames(self, offset=10, limit=10):
        options = Urls.OPTIONS_OFFSET_LIMIT.format(offset, limit)
        url = ''.join([Urls.GAMES, Keys.TOP, options])
        return self._fetchItems(url, Keys.TOP)

    def getGameStreams(self, gameName, offset=10, limit=10):
        quotedName = quote_plus(gameName)
        options = Urls.OPTIONS_OFFSET_LIMIT_GAME.format(offset, limit, quotedName)
        url = ''.join([Urls.BASE, Keys.STREAMS, options])
        return self._fetchItems(url, Keys.STREAMS)

    def searchStreams(self, query, offset=10, limit=10):
        quotedQuery = quote_plus(query)
        options = Urls.OPTIONS_OFFSET_LIMIT_QUERY.format(offset, limit, quotedQuery)
        url = ''.join([Urls.SEARCH, Keys.STREAMS, options])
        return self._fetchItems(url, Keys.STREAMS)

    def getFollowingStreams(self, username):
        #Get ChannelNames
        followingChannels = self.getFollowingChannelNames(username)
        channelNames = self._filterChannelNames(followingChannels)

        #get Streams of that Channels
        options = '?channel=' + ','.join([channels[Keys.NAME] for channels in channelNames])
        url = ''.join([Urls.BASE, Keys.STREAMS, options])
        channels = {'live' : self._fetchItems(url, Keys.STREAMS)}
        channels['others'] = channelNames
        return channels
        
    def getFollowerVideos(self, username, offset, past):
        url = Urls.CHANNEL_VIDEOS.format(username,offset,past)
        items = self.scraper.getJson(url)
        return {Keys.TOTAL : items[Keys.TOTAL], Keys.VIDEOS : items[Keys.VIDEOS]}
        
    def getVideoChunks(self, id):
        url = Urls.VIDEO_CHUNKS.format(id)
        return self.scraper.getJson(url)
        
    def getVideoTitle(self, id):
        url = Urls.VIDEO_INFO.format(id)
        return self._fetchItems(url, 'title')
        
    '''
    needs to be rewritten to return a python list instead of a xbmc playlist
    def getVideoChunksPlaylist(self, id):
        vidChunks = self.getVideoChunks(id)
        chunks = vidChunks['chunks']['live']
        title = self.getVideoTitle(id)
        itemTitle = '%s - Part {0} of %s' % (title, len(chunks))
        
        playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
        playlist.clear()
        
        # For some reason first item is skipped, so added a dummy first item to fix
        # theres probably a better way
        playlist.add('', xbmcgui.ListItem('', thumbnailImage=vidChunks['preview']))
        curN = 0
        for chunk in chunks:
            curN += 1
            playlist.add(chunk['url'], xbmcgui.ListItem(itemTitle.format(curN), thumbnailImage=vidChunks['preview']))
            
        return playlist
    '''
        
    def getFollowingChannelNames(self, username):
        quotedUsername = quote_plus(username)
        url = Urls.FOLLOWED_CHANNELS.format(quotedUsername)
        return self._fetchItems(url, Keys.FOLLOWS)

    def getTeams(self, index):
        return self._fetchItems(Urls.TEAMS.format(str(index * 25)), Keys.TEAMS)

    def getTeamStreams(self, teamName):
        '''
        Consider this method to be unstable, because the
        requested resource is not part of the official Twitch API
        '''
        quotedTeamName = quote_plus(teamName)
        url = Urls.TEAMSTREAM.format(quotedTeamName)
        return self._fetchItems(url, Keys.CHANNELS)

    def _filterChannelNames(self, channels):
        tmp = [{Keys.DISPLAY_NAME : item[Keys.CHANNEL][Keys.DISPLAY_NAME], Keys.NAME : item[Keys.CHANNEL][Keys.NAME], Keys.LOGO : item[Keys.CHANNEL][Keys.LOGO]} for item in channels]
        return sorted(tmp, key=lambda k: k[Keys.DISPLAY_NAME]) 

    def _fetchItems(self, url, key):
        items = self.scraper.getJson(url)
        return items[key] if items else []


class TwitchVideoResolver(object):
    '''
    Resolves the RTMP-Link to a given Channelname
    Uses Justin.TV API
    '''
    
    def __init__(self, logger):
        object.__init__(self)
        self.logger = logger
        self.scraper = JSONScraper(logger)

    def getRTMPUrl(self, channelName, maxQuality):
        swfUrl = self._getSwfUrl(channelName)
        streamQualities = self._getStreamsForChannel(channelName)
        
        self.logger.debug("=== URL and available Streams ===")
        self.logger.debug(json.dumps(swfUrl, sort_keys=True, indent=4))
        
        # check that api response isn't empty (i.e. stream is offline)
        if streamQualities:
            items = [self._parseStreamValues(stream, swfUrl)
                     for stream in streamQualities
                     if self._streamIsAccessible(stream)]
            if items:
                self.logger.debug("=== Accessible Streams ===")
                self.logger.debug(json.dumps(items, sort_keys=True, indent=4))
                return self._bestMatchForChosenQuality(items, maxQuality)[Keys.RTMP_URL]
            else:
                raise TwitchException(TwitchException.NO_STREAM_URL)
        else:
            raise TwitchException(TwitchException.STREAM_OFFLINE)

    def saveHLSToPlaylist(self, channelName, maxQuality, fileName):
        #Get Access Token (not necessary at the moment but could come into effect at any time)
        tokenurl= Urls.CHANNEL_TOKEN.format(channelName)
        channeldata = self.scraper.getJson(tokenurl)
        channeltoken= channeldata['token']
        channelsig= channeldata['sig']

        #Download Multiple Quality Stream Playlist
        data = self.scraper.downloadWebData(Urls.HLS_PLAYLIST.format(channelName,channelsig,channeltoken))

        if "No Results" not in data:
            #Split Into Multiple Lines
            streamurls = data.split('\n')
            #Initialize Custom Playlist Var
            playlist=''
            
            #Define Qualities
            quality = 'Source,High,Medium,Low'
            quality = quality.split(',')
            
            #Initialize Var
            unrestrictedqualities = ''
            #Loop Through Multiple Quality Stream Playlist and Remove Any Restricted Qualities
            for line in range(0, (len(streamurls)-1)):
                if 'EXT-X-TWITCH-RESTRICTED' not in streamurls[line]:
                    unrestrictedqualities += streamurls[line] + '\n'
                    
            streamurls = unrestrictedqualities.split('\n')
            
            self.logger.info('search for quality: ' + quality[maxQuality])
            
            #Check to see if our preferred quality is available (not all qualities are available for none partnered streams)
            if quality[maxQuality] in unrestrictedqualities:
                #Preferred quality is available
                #Loop Through Multiple Quality Stream Playlist Until We Find Our Preferred Quality
                for line in range(0, (len(streamurls)-1)):
                    if quality[maxQuality] in streamurls[line]:
                        #Add Playlist Header
                        playlist = '#EXTM3U\n'
                        #Add 3 Quality Specific Applicable Lines From Multiple Quality Stream Playlist To Our Custom Playlist Var
                        playlist += streamurls[line] + '\n' + streamurls[(line + 1)] + '\n' + streamurls[(line + 2)]
            else:
                #Preferred quality is unavailable so let's play the highest available quality
                playlist += '\n'.join(streamurls)
                self.logger.info("prefered quality unavailable, using highest available quality")
                
            #Write Custom Playlist
            text_file = open(fileName, "w")
            text_file.write(str(playlist))
            text_file.close()

        else:
            raise TwitchException(TwitchException.STREAM_OFFLINE)


    def _getSwfUrl(self, channelName):
        url = Urls.TWITCH_SWF + channelName
        headers = {Keys.USER_AGENT: USER_AGENT,
                   Keys.REFERER: Urls.TWITCH_TV + channelName}
        req = Request(url, None, headers)
        response = urlopen(req)
        return response.geturl()

    def _streamIsAccessible(self, stream):
        stream_is_public = (stream.get(Keys.NEEDED_INFO) != "channel_subscription")
        stream_has_token = stream.get(Keys.TOKEN)

        if stream.get(Keys.CONNECT) is None:
            return False

        return stream_is_public and stream_has_token

    def _getStreamsForChannel(self, channelName):
        scraper = JSONScraper(self.logger)
        url = Urls.TWITCH_API.format(channel=channelName)
        return scraper.getJson(url)

    def _parseStreamValues(self, stream, swfUrl):
        streamVars = {Keys.SWF_URL: swfUrl}
        streamVars[Keys.RTMP] = stream[Keys.CONNECT]
        streamVars[Keys.PLAYPATH] = stream.get(Keys.PLAY)

        if stream[Keys.TOKEN]:
            token = stream[Keys.TOKEN].replace('\\', '\\5c').replace(' ', '\\20').replace('"', '\\22')
        else:
            token = ''

        streamVars[Keys.TOKEN] = (' jtv=' + token) if token else ''
        quality = int(stream.get(Keys.VIDEO_HEIGHT, 0))
        bitrate = int(stream.get(Keys.BITRATE, 0))
        return {Keys.QUALITY: quality,
                Keys.BITRATE: bitrate,
                Keys.RTMP_URL: Urls.FORMAT_FOR_RTMP.format(**streamVars)}

    def _bestMatchForChosenQuality(self, streams, maxQuality):
        # sorting on resolution, then bitrate, both ascending 
        streams.sort(key=lambda t: (t[Keys.QUALITY], t[Keys.BITRATE]))
        self.logger.debug("Available streams sorted: %s" % streams)
        for stream in streams:
            if stream[Keys.QUALITY] <= maxQuality:
                bestMatch = stream
        self.logger.debug("Chosen Stream is: %s" % bestMatch)
        return bestMatch


class Keys(object):
    '''
    Should not be instantiated, just used to categorize
    string-constants
    '''

    BITRATE = 'bitrate'
    CHANNEL = 'channel'
    CHANNELS = 'channels'
    CONNECT = 'connect'
    BACKGROUND = 'background'
    DISPLAY_NAME = 'display_name'
    FEATURED = 'featured'
    FOLLOWS = 'follows'
    GAME = 'game'
    LOGO = 'logo'
    BOX = 'box'
    LARGE = 'large'
    NAME = 'name'
    NEEDED_INFO = 'needed_info'
    PLAY = 'play'
    PLAYPATH = 'playpath'
    QUALITY = 'quality'
    RTMP = 'rtmp'
    STREAMS = 'streams'
    REFERER = 'Referer'
    RTMP_URL = 'rtmpUrl'
    STATUS = 'status'
    STREAM = 'stream'
    SWF_URL = 'swfUrl'
    TEAMS = 'teams'
    TOKEN = 'token'
    TOP = 'top'
    TOTAL = '_total'
    USER_AGENT = 'User-Agent'
    VIDEOS = "videos"
    VIDEO_BANNER = 'video_banner'
    VIDEO_HEIGHT = 'video_height'
    VIEWERS = 'viewers'
    PREVIEW = 'preview'
    TITLE = 'title'


class Patterns(object):
    '''
    Should not be instantiated, just used to categorize
    string-constants
    '''
    VALID_FEED = "^https?:\/\/(?:[^\.]*.)?(?:twitch|justin)\.tv\/([a-zA-Z0-9_]+).*$"
    IP = '.*\d+\.\d+\.\d+\.\d+.*'
    EXPIRATION = '.*"expiration": (\d+)[^\d].*'


class Urls(object):
    '''
    Should not be instantiated, just used to categorize
    string-constants
    '''
    TWITCH_TV = 'http://www.twitch.tv/'

    BASE = 'https://api.twitch.tv/kraken/'
    FOLLOWED_CHANNELS = BASE + 'users/{0}/follows/channels?limit=100'
    GAMES = BASE + 'games/'
    STREAMS = BASE + 'streams/'
    SEARCH = BASE + 'search/'
    TEAMS = BASE + 'teams?limit=25&offset={0}'

    TEAMSTREAM = 'http://api.twitch.tv/api/team/{0}/live_channels.json'
    CHANNEL_TOKEN = 'http://api.twitch.tv/api/channels/{0}/access_token'

    OPTIONS_OFFSET_LIMIT = '?offset={0}&limit={1}'
    OPTIONS_OFFSET_LIMIT_GAME = OPTIONS_OFFSET_LIMIT + '&game={2}'
    OPTIONS_OFFSET_LIMIT_QUERY = OPTIONS_OFFSET_LIMIT + '&q={2}'

    TWITCH_API = "http://usher.justin.tv/find/{channel}.json?type=any&group=&channel_subscription="
    TWITCH_SWF = "http://www.justin.tv/widgets/live_embed_player.swf?channel="
    FORMAT_FOR_RTMP = "{rtmp}/{playpath} swfUrl={swfUrl} swfVfy=1 {token} live=1"  # Pageurl missing here
    HLS_PLAYLIST = 'http://usher.twitch.tv/select/{0}.m3u8?nauthsig={1}&nauth={2}&allow_source=true'
    
    CHANNEL_VIDEOS = 'https://api.twitch.tv/kraken/channels/{0}/videos?limit=8&offset={1}&broadcasts={2}'
    VIDEO_CHUNKS = 'https://api.twitch.tv/api/videos/{0}'
    VIDEO_INFO = 'https://api.twitch.tv/kraken/videos/{0}'
        


class TwitchException(Exception):

    NO_STREAM_URL = 0
    STREAM_OFFLINE = 1
    HTTP_ERROR = 2
    JSON_ERROR = 3

    def __init__(self, code):
        Exception.__init__(self)
        self.code = code

    def __str__(self):
        return repr(self.code)



#-*- encoding: utf-8 -*-
#Note: The YouTube Data API (v2) has been officially deprecated as of March 4, 2014. Please refer to our deprecation policy for more information.
#https://developers.google.com/youtube/2.0/developers_guide_protocol_api_query_parameters

try:
    from urllib.request import urlopen, Request
    from urllib.parse import quote_plus, quote
except ImportError:
    from urllib import quote_plus, quote
    from urllib2 import Request, urlopen

import logging
import sys
import xml.etree.ElementTree as et

USER_AGENT = 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:6.0) Gecko/20100101 Firefox/6.0'

class XMLScraper():
    __logger=logging.getLogger(__name__)
    
    def downloadWebData(self, url):
        data = ""
        try:
            req = Request(url)
            req.add_header('User-Agent', USER_AGENT)
            response = urlopen(req)
            
            if sys.version_info < (3, 0):
                data = response.read()
            else:
                data = response.readall().decode('utf-8')
            response.close()
        except:
            raise Exception()
        return data
    
    def downloadXML(self, url):
        try:
            xmlString = self.downloadWebData(url)
            tree = et.fromstring(xmlString)
            self.__logger.debug(et.dump(tree))
            return tree
        except:
            raise Exception()
    
    
class Youtube():
    __scraper = None
    __logger=logging.getLogger(__name__)
    NAMESPACES = {
        'Atom': 'http://www.w3.org/2005/Atom',
        'openSearch': 'http://a9.com/-/spec/opensearchrss/1.0/'}
    URLS = {
        'uploads'      : 'https://gdata.youtube.com/feeds/api/users/{0[0]}/uploads?start-index={0[1]}&max-results={0[2]}',
        'channelSearch': 'https://gdata.youtube.com/feeds/api/channels?q="{0[0]}"&start-index={0[1]}&max-results={0[2]}&v=2',
        'videoSearch': 'https://gdata.youtube.com/feeds/api/videos?q="{0[0]}"&start-index={0[1]}&max-results={0[2]}&v=2'}
    
    def __init__(self):
        self.__scraper=XMLScraper()
        
    def __prepareURL(self, url_key, replacement_list):
        url = self.URLS[url_key].format(replacement_list)
        url = quote(url, safe="%/:=&?~#+!$,;'@()*[]")
        self.__logger.debug('prepareURL: ' + url)
        return url
        
    '''
        list all Channel videos
        @param channelName name of the channel
        @param offset where to start(smallest is 1)
        @param limit max amount of videos to pull, max limit is 25
    '''
    def listChannelVideos(self, channelName, offset=1, limit=25):
        url = self.__prepareURL('uploads', (channelName, offset, limit))
        return self.__scraper.downloadXML(url)
        
    '''
        search for Channel
        @param channelName name of the channel
        @param offset where to start(smallest is 1)
        @param limit max amount of videos to pull, max limit is 25
    '''
    def searchChannel(self, channelName, offset=1, limit=25):
        url = self.__prepareURL('channelSearch', (channelName, offset, limit))
        return self.__scraper.downloadXML(url)
        
    '''
        search for Channel
        @param videoName name of the video
        @param offset where to start(smallest is 1)
        @param limit max amount of videos to pull, max limit is 25
    '''
    def searchVideo(self, videoName, offset=1, limit=25):
        url = self.__prepareURL('videoSearch', (videoName, offset, limit))
        return self.__scraper.downloadXML(url)
        



import logging

class Interface():
    pass
    #no idea what i need to abstract here, need to revisit when adding console interface
    
'''
    provides an interface for objects to be queried by the UI
'''
class Displayable():
    def getDict(self):
        raise NotImplementedError
        
    def getName(self):
        return self.__class__.__name__
        
        
'''
    provides feedback to the user if an exception occurs
'''
class ExceptionDisplayHandler(Displayable):
    __logger=logging.getLogger(__name__)
    __head=""
    __body=""
    __status=0

    def __init__(self):
        pass
        
    '''
        set an Exception that is to be displayed by javascript in a client window
        @param head Header of the Exception
        @body body text of th eException
    '''
    def setException(self, head, body):
        self.__logger.info('setException with head: ' + head + ' and body: ' + body)
        self.__head=head
        self.__body=body
        self.__status=1
        
    '''
        clears the Exception from the System
    '''
    def clearException(self):
        self.__logger.info('clearing Exception')
        self.__head=""
        self.__body=""
        self.__status=0
        
    #@override
    def getDict(self):
        tempDict={}
        tempDict['exceptionStatus'] = self.__status
        tempDict['exceptionTitle'] = self.__head
        tempDict['exceptionBody'] = self.__body
        return tempDict
        
'''
    lists in the interface, for example players/playlists/browsers
'''
class InterfaceListable():
    __array=None
    __current=None

    def __init__(self, array):
        if(array is None):
            raise TypeError('array is not allowed to be None')
            
        if(len(array)<0):
            raise TypeError('array is not allowed to be smaller then 0')
            
        if(len(array)<=0):
            self.__array=None
            self.__current=None
        else:
            self.__array=array
            self.__current=0
        
    def setActive(self, id):
        if(self.__array is None):
            return None
        if(0<=id<len(self.__array)):
            self.__current=id
        else:
            raise TypeError('id not in range of playerArray with size ' + len(self.__array))
            
    def getActive(self):
        if(self.__array is None):
            return None
        return self.__array[self.__current]
        
    def getIDfromName(self, name):
        tempid=0
        for entry in self.getArray():
            if(entry==name):
                return tempid
            else:
                tempid = tempid + 1
        return None
        
    def getArray(self):
        if(self.__array is None):
            return None
        temparray = []
        for i in self.__array:
            temparray.append(i.getName())
        return temparray



from simpleMediaCenter.player.Player import Player
import logging
import subprocess
import shlex
import time

class Omxplayer(Player):
    __logger=logging.getLogger(__name__)
    __cmdline=""
    __playerline="omxplayer -b"
    __playerstatus=0 # 0 stopped, 1 playing, 2 paused
    __currentfile=""
    __process=None
    
    keymapping = {
        'pause' : 'p',
        'quit'  : 'q',
        'volume': {
                    'up'   : '+',
                    'down' : '-'
                  }
        }
    

    def __init__(self, cmdline = "-o both"):
        self.__logger.debug("Omxplayer init")
        self.cmdline=cmdline
        
    def __resetplayer(self):
        self.__logger.debug("reset Player")
        self.__playerstatus=0
        self.__currentfile=""
        self.__paused=False
        self.__process=None
        
    def poll(self):
        self.__logger.debug("polling")
        if(self.__process is not None):
            self.__logger.debug("process is not none")
            self.__process.poll()
            if(self.__process.returncode is not None):
                self.__logger.debug("process ended")
                for line in self.__process.stdout:
                    self.__logger.debug("stdout" + line.decode('utf-8'))
                self.__resetplayer()
                
    
    def send(self, str):
        self.__logger.debug("Player send %s", str)
        self.poll()
        if(self.__process is not None):
            self.__process.stdin.write(bytes(str, 'UTF-8'))
            self.__process.stdin.flush()
            
    def getcmdline(self,file):
        return self.__playerline + " " + self.__cmdline + " '" + file + "'"

    def play(self, file):
        self.poll()
        self.__logger.debug("playing: %s",file)
        if(self.__process is not None):
            self.stop()
            
        line = self.getcmdline(file)
        self.__process = subprocess.Popen(
            shlex.split(line), 
            stdout=subprocess.PIPE, 
            stdin=subprocess.PIPE, 
            stderr=subprocess.STDOUT,
            close_fds=True)
        self.__playerstatus=1
        self.__currentfile = file
        self.__paused=False
        
    def pause(self):
        self.poll()
        self.__logger.debug("pause called")
        if(self.__process is not None):
            self.send(self.keymapping['pause'])
            if(self.__playerstatus==1):
                self.__playerstatus=2
            else:
                self.__playerstatus=1
        
    def stop(self):
        self.poll()
        self.__logger.debug("stopping")
        if(self.__process is not None):
            self.send(self.keymapping['quit'])
            try:
                self.__logger.debug("waiting for process to close")
                self.__process.wait(timeout=5)
            except subprocess.TimeoutExpired:
                self.__logger.debug("timeout occured, killing")
                #self.__process.kill()
                subprocess.Popen(shlex.split("killall omxplayer.bin")).wait() ##quickhack
            self.__resetplayer()
            self.__logger.debug("player stopped")
            
    def volumeUp(self):
        self.poll()
        self.__logger.debug("increase vol")
        if(self.__process is not None):
            self.send(self.keymapping['volume']['up'])
        
    def volumeDown(self):
        self.poll()
        self.__logger.debug("decrease vol")
        if(self.__process is not None):
            self.send(self.keymapping['volume']['down'])
        
    def getDict(self):
        self.poll()
        tempDict={}
        tempDict['activePlayer'] = self.getName()
        tempDict['playerStatus'] = self.__playerstatus
        tempDict['currentFile'] = self.__currentfile
        return tempDict
    



#needs rework, pathing needs to be done via a full path, not just parent dir
#similar to a rest interface would be best 
#/ marks root
#/Games/Game Name/1 could display the first page and so on <- workingDir
'''
>>> path = "/Games/Garry's Mod/1"
>>> path.split('/')
['', 'Games', "Garry's Mod", '1']
>>> sp = path.split('/')
>>> '/'.join(sp)
"/Games/Garry's Mod/1"
>>>

'''

import os
import logging
from simpleMediaCenter.interface.Interface import Displayable


class Browser(Displayable):
    workingDir = ''
    dirlist = {}
    filelist = {}
    
    def __init__(self, startDirectory):
        self.setWorkingDir(startDirectory)
       
    '''
        Returns a String that can be interpreted by a Player to play a ressource 
        (filePath for omxplayer, channelname for twitchplayer,...)
    '''    
    def getPlayable(self, fileKey):
        raise NotImplementedError
        
    ''' 
        Returns a String that represents a Path (Directory, or menu level)
    '''    
    def getPath(self, pathKey):
        raise NotImplementedError
      
    '''
        get supported Players, first match gets used to play file returned by getPlayable
    '''
    def getSupportedPlayers(self):
        raise NotImplementedError
    
    '''
        Set new Working Dir 
        @param newWorkingDirKey represents a Key in dirlist
    '''    
    def setWorkingDir(self, newWorkingDirKey):
        raise NotImplementedError
      
    def getWorkingDir(self):
        return self.workingDir
        
    def getPathList(self):
        return self.dirlist
        
    def getFileList(self):
        return self.filelist
        
    def getDict(self):
        tempDict={}
        tempDict['browserWorkingDir'] = self.getWorkingDir()
        tempDict['browserDirs'] = self.getPathList()
        tempDict['browserFiles']= self.getFileList()
        tempDict['activeBrowser'] = self.getName()
        tempDict['browserSearch'] = False
        return tempDict
    
    def getName(self):
        return self.__class__.__name__
        

class Pagination():
    minimum = 0
    offset = 0
    limit = 0
    nextPageString = 'next Page >'
    prevPageString = '< previous Page'

    def __init__(self, startoffset, limit):
        self.offset = startoffset
        self.minimum = startoffset
        self.limit = limit
        
    def increase(self):
        self.offset = self.offset + self.limit
        
    def decrease(self):
        if(self.offset - self.limit >= self.minimum):
            self.offset = self.offset - self.limit
            
    def reset(self):
        self.offset = self.minimum


class FileBrowser(Browser):    
    __logger=logging.getLogger(__name__)

    def __init__(self,startDirectory='~'):
        self.__logger.debug('init')
        tempDir = os.path.expanduser(startDirectory)
        tempDir = os.path.abspath(tempDir)
        self.dirlist = {
                0 : tempDir
            }
        self.setWorkingDir(0)
    
    def getPlayable(self, fileKey):
        temp = os.path.join(self.workingDir,self.filelist[fileKey])
        self.__logger.debug('getPlayable:' + temp)
        return temp
        
    def getPath(self, pathKey):
        tempPath = os.path.join(self.workingDir,self.dirlist[pathKey])
        tempPath = os.path.abspath(tempPath)
        self.__logger.debug('getPath:' + tempPath)
        return tempPath
      
    def getSupportedPlayers(self):
        return ['Omxplayer', 'MPlayer']
      
    def setWorkingDir(self, newWorkingDirID):
        self.workingDir = self.getPath(newWorkingDirID)
        list = os.listdir(self.workingDir)
        
        dirlistcounter=0
        filelistcounter=0
        self.dirlist = {}
        self.filelist = {}
        
        self.dirlist[dirlistcounter] = '.'
        dirlistcounter+=1
        
        self.dirlist[dirlistcounter] = '..'
        dirlistcounter+=1
        
        for entry in list:
            testentry = os.path.join(self.workingDir,entry)
            if(os.path.isfile(testentry)):
                self.filelist[filelistcounter] = entry
                filelistcounter+=1
            else:
                self.dirlist[dirlistcounter] = entry
                dirlistcounter+=1

                
class TwitchBrowser(Browser):
    __logger=logging.getLogger(__name__)
    pagination = None
    twitchTV = None
    username = None
    
    def __init__(self, username=None):
        from simpleMediaCenter.helpers.twitch import TwitchTV
        self.twitchTV = TwitchTV(self.__logger)
        self.pagination = Pagination(startoffset=0,limit=10)
        
        self.username = username
        self.dirlist = {
                0 : '.'
            }
        self.setWorkingDir(0)
        
    def getPlayable(self, fileKey):
        self.__logger.debug('getPlayable:' + self.filelist[fileKey])
        return self.filelist[fileKey]
        
    def splitPath(self, path):# take away root / and split by '/', return empty array if on root
        if(path=='/'):
            return []
        if(path==''):
            return []
        return path[1:].split('/')
        
    def buildPath(self, patharray):
        return '/' + '/'.join(patharray)
       
    '''
        returns a split path array 
        >>> tempPath
        ['menu','submenu']
        ['']
    '''
    def getPath(self, pathKey):
        tempPath = self.dirlist[pathKey]
        oldWorkingDir = self.splitPath(self.getWorkingDir())
        self.__logger.debug("oldWorkingDir: " + self.getWorkingDir())
        self.__logger.debug('getPath, tempPath:' + tempPath)
        if(tempPath=='.'):
            tempPath=oldWorkingDir
        elif(tempPath=='..'):
            if(len(oldWorkingDir)>0): # check if not on top level
                del oldWorkingDir[-1] # delete last element in list
            else:
                oldWorkingDir = []
            tempPath = oldWorkingDir
            self.pagination.reset()
        elif(tempPath==self.pagination.nextPageString):
            self.pagination.increase()
            tempPath=oldWorkingDir
        elif(tempPath==self.pagination.prevPageString):
            self.pagination.decrease()
            tempPath=oldWorkingDir
        else:
            tempPath = oldWorkingDir + [tempPath]
            self.pagination.reset()
        return tempPath

    def getSupportedPlayers(self):
        return ['Twitchplayer']
    
    def setWorkingDir(self, newWorkingDirID):
        self.__logger.debug('setWorkingDir id: ' + repr(newWorkingDirID))
        
        pathArray = self.getPath(newWorkingDirID)
        
        dirlistcounter=0
        filelistcounter=0
        self.dirlist = {}
        self.filelist = {}
       
        self.__logger.debug("setWorkingDir, | seperated: " + "|".join(pathArray))

        '''
            parse every element of path array to find destination
        '''
        if (len(pathArray)==0):
            self.dirlist[dirlistcounter] = 'Featured'
            dirlistcounter+=1
            self.dirlist[dirlistcounter] = 'Games'
            dirlistcounter+=1
            self.dirlist[dirlistcounter] = 'Following'
            dirlistcounter+=1
            
        elif (len(pathArray)>=1):
            if (pathArray[0]=='Featured'):
                self.dirlist[dirlistcounter] = '.'
                dirlistcounter+=1
            
                self.dirlist[dirlistcounter] = '..'
                dirlistcounter+=1
                
                featured = self.twitchTV.getFeaturedStream()
                for stream in featured:
                    self.filelist[filelistcounter] = stream['stream']['channel']['name']
                    filelistcounter+=1
                    
            elif (pathArray[0]=='Games'):   
                if(len(pathArray)==1):
                    self.dirlist[dirlistcounter] = '.'
                    dirlistcounter+=1
                
                    self.dirlist[dirlistcounter] = '..'
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = self.pagination.prevPageString
                    dirlistcounter+=1
                    
                    self.__logger.debug('offset:' + str(self.pagination.offset) + ' limit:' + str(self.pagination.limit))
                    games = self.twitchTV.getGames(offset=self.pagination.offset, limit=self.pagination.limit)
                    for game in games:
                        self.dirlist[dirlistcounter] = game['game']['name']
                        dirlistcounter+=1
                    self.dirlist[dirlistcounter] = self.pagination.nextPageString
                    dirlistcounter+=1
                elif(len(pathArray)==2):
                    self.__logger.debug("list channels which play game: " + pathArray[1])
                    
                    self.dirlist[dirlistcounter] = '.'
                    dirlistcounter+=1
                
                    self.dirlist[dirlistcounter] = '..'
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = self.pagination.prevPageString
                    dirlistcounter+=1
                    
                    gamestreams = self.twitchTV.getGameStreams(gameName=pathArray[1], offset=self.pagination.offset, limit=self.pagination.limit)
                    for stream in gamestreams:
                        self.filelist[filelistcounter] = stream['channel']['name']
                        filelistcounter+=1
                        
                    self.dirlist[dirlistcounter] = self.pagination.nextPageString
                    dirlistcounter+=1
                
            elif (pathArray[0]=='Following'):
                self.dirlist[dirlistcounter] = '.'
                dirlistcounter+=1
            
                self.dirlist[dirlistcounter] = '..'
                dirlistcounter+=1
                
                if(self.username is None):
                    return False # temporary workaround if no username is set
                
                following = self.twitchTV.getFollowingStreams(self.username)
                for stream in following['live']:
                    self.filelist[filelistcounter] = stream['channel']['name']
                    filelistcounter+=1

            else:
                self.__logger.critical('no suiting menu found workingDir: ' + "|".join(self.workingDir))
                raise NotImplementedError
            
        
        self.workingDir = self.buildPath(pathArray)
        self.__logger.debug("setWorkingDir to: " + self.workingDir)

        
    def setUsername(self, username):
        self.username = username
        
    def getName(self):
        if(self.username is None):
            return self.__class__.__name__
        return self.__class__.__name__ + " user: " + self.username
    
    
class YoutubeBrowser(Browser):
    __logger=logging.getLogger(__name__)
    favorites = []
    pagination = None
    urllist = {}
    yt = None

    def __init__(self):
        from simpleMediaCenter.helpers.youtube import Youtube
        self.yt = Youtube()
        self.pagination = Pagination(startoffset=1,limit=10)
        self.dirlist = {
                0 : '.'
            }
        self.setWorkingDir(0)
        
    def getPlayable(self, fileKey):
        return self.urllist[fileKey]
        
    def getPath(self, pathKey):
        tempPath = self.dirlist[pathKey]
        oldWorkingDir = self.splitPath(self.getWorkingDir())
        self.__logger.debug("oldWorkingDir: " + self.getWorkingDir())
        self.__logger.debug('getPath, tempPath:' + tempPath)
        if(tempPath=='.'):
            tempPath=oldWorkingDir
        elif(tempPath=='..'):
            if(len(oldWorkingDir)>0): # check if not on top level
                del oldWorkingDir[-1] # delete last element in list
            else:
                oldWorkingDir = []
            tempPath = oldWorkingDir
            self.pagination.reset()
        elif(tempPath==self.pagination.nextPageString):
            self.pagination.increase()
            tempPath=oldWorkingDir
        elif(tempPath==self.pagination.prevPageString):
            self.pagination.decrease()
            tempPath=oldWorkingDir
        elif(tempPath=='/'):
            tempPath = []
        else:
            tempPath = oldWorkingDir + [tempPath]
            self.pagination.reset()
        return tempPath

    def getSupportedPlayers(self):
        return ['Youtubeplayer']
        
    def splitPath(self, path):# take away root / and split by '/', return empty array if on root
        if(path=='/'):
            return []
        if(path==''):
            return []
        return path[1:].split('/')
        
    def buildPath(self, patharray):
        return '/' + '/'.join(patharray)
    
    def setWorkingDir(self, newWorkingDirID, search=None):
        self.__logger.debug('setWorkingDir in YoutubeBrowser, ' + repr(newWorkingDirID))
        if(search==None):
            pathArray = self.getPath(newWorkingDirID)
        else:
            pathArray = ['search',search,newWorkingDirID]
        
        dirlistcounter=0
        filelistcounter=0
        self.dirlist = {}
        self.filelist = {}
        self.urllist = {}
       
       
       
        self.__logger.debug("setWorkingDir, final: " + self.workingDir)
        
        if (len(pathArray)==0):
            self.dirlist[dirlistcounter] = 'Favorites' #TODO 
            dirlistcounter+=1
        
        elif (len(pathArray)>=1):
            if (pathArray[0]=='Favorites'):
                if(len(pathArray)==1):
                    self.dirlist[dirlistcounter] = '..'
                    dirlistcounter+=1
                    
                    for fav in self.favorites:
                        self.dirlist[dirlistcounter] = fav
                        dirlistcounter+=1

                elif(len(pathArray)==2):
                    self.dirlist[dirlistcounter] = '.'
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = '..'
                    dirlistcounter+=1
                    
                    self.__logger.debug('getting videolist')
                    videos = self.yt.listChannelVideos(pathArray[1], offset=self.pagination.offset, limit=self.pagination.limit)
                    self.__logger.debug('searching videolist')
                    for video in videos.findall('Atom:entry', namespaces=self.yt.NAMESPACES):
                        self.urllist[filelistcounter] = video.find(
                            ".//Atom:link[@rel='alternate']", 
                            namespaces=self.yt.NAMESPACES).get('href')
                        self.filelist[filelistcounter] = video.find(
                            'Atom:title',
                            namespaces=self.yt.NAMESPACES).text
                        filelistcounter+=1
                else:
                    self.__logger.critical('no suiting menu found workingDir: ' + "|".join(self.workingDir))
                    raise NotImplementedError

            elif (pathArray[0]=='search'):
                if(len(pathArray)==3):
                    self.__logger.debug("search for " + pathArray[1] + " " + pathArray[2])
                    self.dirlist[dirlistcounter] = '.'
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = '/'
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = self.pagination.prevPageString
                    dirlistcounter+=1
                    
                    self.dirlist[dirlistcounter] = self.pagination.nextPageString
                    dirlistcounter+=1
                    
                    if(pathArray[1]=='File'):                    
                        videos = self.yt.searchVideo(pathArray[2], offset=self.pagination.offset, limit=self.pagination.limit)
                        for video in videos.findall('Atom:entry', namespaces=self.yt.NAMESPACES):
                            self.urllist[filelistcounter] = video.find(
                                ".//Atom:link[@rel='alternate']", 
                                namespaces=self.yt.NAMESPACES).get('href')
                            title = video.find('Atom:title',namespaces=self.yt.NAMESPACES).text
                            title = title + " - Channel:" + video.find('Atom:author/Atom:name',namespaces=self.yt.NAMESPACES).text
                            self.filelist[filelistcounter] = title
                            filelistcounter+=1
               
                    elif (pathArray[1]=='Dir'):    
                        videos = self.yt.searchVideo(pathArray[2], offset=self.pagination.offset, limit=self.pagination.limit)
                        for video in videos.findall('Atom:entry', namespaces=self.yt.NAMESPACES):
                            title = video.find('Atom:author/Atom:name',namespaces=self.yt.NAMESPACES).text
                            self.dirlist[dirlistcounter] = title
                            dirlistcounter+=1
                    else:
                        self.__logger.critical('no suiting menu found workingDir: ' + "|".join(self.workingDir))
                        raise NotImplementedError
                        
            else:
                self.__logger.critical('no suiting menu found workingDir: ' + "|".join(self.workingDir))
                raise NotImplementedError
        else:
            self.__logger.critical('no suiting menu found workingDir: ' + "|".join(self.workingDir))
            raise NotImplementedError
                
            
        self.workingDir = self.buildPath(pathArray)
        self.__logger.debug("setWorkingDir to: " + self.workingDir)
        
    def addFavorite(self, newFavorite):
        self.favorites.append(newFavorite)
       
    def getDict(self):
        tempDict=super().getDict()
        tempDict['browserSearch'] = True
        return tempDict


from simpleMediaCenter.interface.Interface import Interface, ExceptionDisplayHandler
from simpleMediaCenter.helpers.twitch import TwitchException
from tg import expose, TGController, AppConfig, redirect, config
from wsgiref.simple_server import make_server, WSGIServer
from socketserver import ThreadingMixIn
import os
import jinja2
import logging
import json

class WebController(TGController):  
    __logger=logging.getLogger(__name__)
    playerList=None
    playlistList=None
    browserList=None
    statusDict=None
    exceptionDisplayHandler=None

    def __init__(self, playerList=None, playlistList=None, browserList=None ):
        self.__logger.debug("WebInterface init")
        self.playerList=playerList
        self.browserList=browserList
        self.playlistList=playlistList
        self.exceptionDisplayHandler=ExceptionDisplayHandler()
        self.statusDict = {
            'playerArray'  : self.playerList.getArray(),
            'playlistArray'  : self.playlistList.getArray(),
            'browserArray'  : self.browserList.getArray(),
            }
            
    def updateStatus(self):
        self.__logger.debug("updateStatus")
        if(self.playerList.getActive() is not None):
            self.statusDict.update(self.playerList.getActive().getDict())
        if(self.playlistList.getActive() is not None):
            self.statusDict.update(self.playlistList.getActive().getDict())
        if(self.browserList.getActive() is not None):
            self.statusDict.update(self.browserList.getActive().getDict())
        if(self.exceptionDisplayHandler is not None):
            self.statusDict.update(self.exceptionDisplayHandler.getDict())
            
        
    def parseID(self, id):
        id = int(id)
        return id
    
    #index Page
    @expose('index.html')
    def index(self):
        self.__logger.debug('index called')
        self.updateStatus()
        self.__logger.debug(self.statusDict)
        return self.statusDict
       
    '''
    #loading parking Page
    @expose('loading.html')
    def loading(self):
        self.__logger.debug('loading called')
        redirect("/")
    '''    
        
    
    #controls
    '''
        play file, auto select suitable player
    '''
    @expose()
    def play(self, id=None):
        try:
            id = self.parseID(id)
            self.__logger.debug("trying to play %s" ,self.browserList.getActive().getPlayable(id))
            self.__logger.debug("searching for compatible Browser") 
            for supportedPlayer in self.browserList.getActive().getSupportedPlayers():
                playerid = self.playerList.getIDfromName(supportedPlayer)
                if (playerid is not None):
                    break
            if (playerid is None):
                raise Exception('no compatible player found in supportedPlayers')
            self.playerList.getActive().stop()
            self.playerList.setActive(playerid)
            self.playerList.getActive().play(self.browserList.getActive().getPlayable(id))
        except Exception as e:
            self.exceptionDisplayHandler.setException('Exception','Unhandled Exception in play: ' + repr(e))

        redirect("/")
    
    '''
        stop playback
    '''
    @expose()
    def stop(self):
        self.__logger.debug("stop called")
        self.playerList.getActive().stop()
        redirect("/")
      
    '''
        pause playback
    '''
    @expose()
    def pause(self):
        self.__logger.debug("pause called")
        self.playerList.getActive().pause()
        redirect("/")
        
    '''
        reduces player volume
    '''
    @expose()
    def volumedown(self):
        self.__logger.debug("volume down called")
        self.playerList.getActive().volumeDown()
        redirect("/")
        
    '''
        increases player volume
    '''
    @expose()
    def volumeup(self):
        self.__logger.debug("volume up called")
        self.playerList.getActive().volumeUp()
        redirect("/")
        
    '''
        change working directory
    '''
    @expose()
    def change(self,id=None):
        try:
            id = self.parseID(id)
            self.__logger.debug("change called %s", id)
            self.browserList.getActive().setWorkingDir(id)
        except KeyError as e:
            self.exceptionDisplayHandler.setException('KeyError','Passed id not in range: ' + repr(e))
        except TwitchException as e:
            self.exceptionDisplayHandler.setException('TwitchException',repr(e))
        except Exception as e:
            self.exceptionDisplayHandler.setException('Exception','Unhandled Exception in change: ' + repr(e))
        redirect("/")
        
        
    '''
        search something which is then treated as a playable file
    '''    
    @expose()
    def searchFile(self, search):
        self.__logger.debug("searchFile called")
        #check if browser has search extension enabled, KEY ERROR
        if(self.browserList.getActive().getDict()['browserSearch']==True):
            self.browserList.getActive().setWorkingDir(search, search='File')
        redirect("/")
        
    '''
        search something which is then treated as a browsable directory
    '''
    @expose()
    def searchDir(self, search):
        self.__logger.debug("searchDir called")
        #check if browser has search extension enabled, KEY ERROR
        if(self.browserList.getActive().getDict()['browserSearch']==True):
            self.__logger.debug("browser Search enabled")
            self.browserList.getActive().setWorkingDir(search, search='Dir')
        redirect("/")
      
    '''
        DEPRECATED: is now done automatically
        selects player via dropdown menu
    '''      
    @expose()
    def selectPlayer(self, id=None):
        try:
            id = self.parseID(id)
            self.playerList.getActive().stop()
            self.playerList.setActive(id)
        except Exception as e:
            self.exceptionDisplayHandler.setException('Exception','Unhandled Exception in selectPlayer: ' + repr(e))
        redirect("/")
        
    '''
        selects browser via dropdown menu
    '''
    @expose()
    def selectBrowser(self, id=None):
        try:
            id = self.parseID(id)
            self.browserList.setActive(id)
        except Exception as e:
            self.exceptionDisplayHandler.setException('Exception','Unhandled Exception in selectBrowser: ' + repr(e))
        redirect("/")
        
    '''
        selects playlist via dropdown menu
    '''
    @expose()
    def selectPlaylist(self, id=None):
        try:
            id = self.parseID(id)
            self.playlistList.setActive(id)
        except Exception as e:
            self.exceptionDisplayHandler.setException('Exception','Unhandled Exception in selectPlaylist: ' + repr(e))
        redirect("/")
        
    '''
        clears an exception from memory
    '''
    @expose()
    def clearException(self):
        self.exceptionDisplayHandler.clearException()
        redirect("/")
        
    #ajax interface
    @expose()
    def status(self,_=None):
        self.updateStatus()
        return json.dumps(self.statusDict)

        
class ThreadingWSGIServer(ThreadingMixIn, WSGIServer): 
    pass

class WebInterface(Interface):
    __logger=logging.getLogger(__name__)
    config=None
    httpd=None
    application=None
    keep_running=True
    player=None
    controller=None
    
    def __init__(self, templatePath, staticPath, port, controller):
        self.__logger.debug("create WebInterface Instance")
  
        self.__logger.debug("setup TurboGears2")
        self.controller = controller
        self.config = AppConfig(minimal=True, root_controller=self.controller)
        
        #jinja stuff
        self.config.renderers.append('jinja')
        self.config.default_renderer = 'jinja'
        self.config.use_dotted_templatenames = False
        self.config.paths['templates'] = [templatePath]
        
        #statics
        self.config.serve_static = True
        self.config.paths['static_files'] = staticPath
        
        #make wsgi_app      
        self.application = self.config.make_wsgi_app()
        
        #make wsgi_server
        self.httpd = make_server('', port, self.application, ThreadingWSGIServer)
        self.httpd.timeout = 5
    
    def run(self):
        self.__logger.debug("run")
        while(self.keep_running):
            self.__logger.info("listening for Requests")
            try:
                self.httpd.serve_forever()
            except KeyboardInterrupt:
                self.shutdown()
    
    def shutdown(self):
        self.__logger.info("init shutdown")
        self.controller.playerList.getActive().stop()
        self.keep_running=False



from simpleMediaCenter.player.Omxplayer import Omxplayer
import youtube_dl
import logging
import sys
import os

class Youtubeplayer(Omxplayer):
    __logger=logging.getLogger(__name__)
    __playerline="omxplayer -b --live"
    __cmdline="-o both"
    __ytdl=None

    def __init__(self, cmdline = "-o both"):
        self.__logger.debug("Youtubeplayer init")
        self.__cmdline=cmdline
        self.__ytdl=youtube_dl.YoutubeDL({'outtmpl': '%(id)s.%(ext)s'})
        self.__ytdl.add_default_info_extractors()
  
    def getcmdline(self,url):
        result = self.__ytdl.extract_info(url,download=False)
        #insert resultcheck here
        cmdline = self.__playerline + " " + self.__cmdline + " '" + result['url'] + "'"
        self.__logger.debug('Youtubeplayer cmdline: ' + cmdline)
        return cmdline




import os
import logging
from collections import deque
from simpleMediaCenter.interface.Interface import Displayable

class Playlist(Displayable):
    #list of some sort
    def __init__(self):
        pass

    def add(self, filePath):
        raise NotImplementedError
        
    def getNext(self ):
        raise NotImplementedError

class FiFo(Playlist):
    queue = deque([])
    
    def __init__(self):
        pass

    def add(self, filePath):
        logging.debug("try to add: %s", filePath)
        if(os.path.isfile(filePath)):
            self.queue.append(filePath)
        else:
            raise Exception
        
    def getNext(self):
        logging.debug("getNext called")
        if(len(self.queue) > 0):
            return self.queue.popleft()
        else:
            raise Exception
    
    def getDict(self):
        raise NotImplementedError
            
class Single(Playlist):
    filePath = None
    
    def __init__(self):
        pass
    
    def add(self, filePath):
        self.filePath=filePath
        
    def getNext(self):
        return None
  
    def getDict(self):
        tempDict={}
        tempDict['activePlaylist'] = self.getName()
        tempDict['playlistFiles'] = [self.filePath]
        return tempDict
        
    def getName(self):
        return self.__class__.__name__
        



from distutils.core import setup

upstream = 'https://github.com/ingwinlu/simpleMediaCenter'
devblog = 'http://heroicdebugging.biz'

setup(
    name='simpleMediaCenter',
    description='simpleMediaCenter aims to provide an easy on ressources way to use your computer as an Media Center. It is primarily designed to act as an alternative to XBMC on the rpi.',
    version='0.6.1',
    url=upstream,
    packages=['simpleMediaCenter',
            'simpleMediaCenter.browser',
            'simpleMediaCenter.helpers',
            'simpleMediaCenter.helpers.twitch',
            'simpleMediaCenter.helpers.youtube',
            'simpleMediaCenter.interface',
            'simpleMediaCenter.player',
            'simpleMediaCenter.playlist'],
    package_data={
            'simpleMediaCenter.interface' : ['templates/*.html','static/*.css','static/*.js'],
            '' : ['LICENSE','README.md']
    },
    scripts=['simpleMC'],
    requires = [
        'jinja2',
        'turbogears2',
        'youtube_dl'
    ],
    license='GPLv3',
    long_description='For a longer description, please have a look at ' + upstream + ' or the development blog at ' + devblog,
    author='winlu',
    author_email='derwinlu+simpleMediaCenter@gmail.com',
)




from simpleMediaCenter.interface.Interface import Displayable

class Player(Displayable):

    def __init__(self, cmdline):
        raise NotImplementedError

    def send(self, str):
        raise NotImplementedError

    def play(self, file):
        raise NotImplementedError
        
    def pause(self):
        raise NotImplementedError
        
    def stop(self):
        raise NotImplementedError
        
    def volumeUp(self):
        raise NotImplementedError
        
    def volumeDown(self):
        raise NotImplementedError
    
    def getName(self):
        return self.__class__.__name__

'''    
overwrite init and check for arraycontent for type=Player
class PlayerList(InterfaceListable):
    self.__playerArray=None
    __currentPlayer=0

    def __init__(self, playerArray):
        if(playerArray is None):
            raise AttributeError('playerArray is not allowed to be None')
            
        if(len(playerArray)<=0):
            raise AttributeError('playerArray is not allowed to be of size 0 or smaller')
            
        self.__PlayerArray=playerArray
        self.__currentPlayer=0
        
    def setActive(self, id):
        if(0<=id<len(self.__playerArray)):
        
        else:
            raise AttributeError('id not in range of playerArray with size ' + len(self.__playerArray))
            
    def getActive(self):
        return self.__playerArray[self.__currentPlayer]
'''